"""pypickle is to save and load variables in/from pickle files."""
# --------------------------------------------------
# Name        : pypickle.py
# Author      : E.Taskesen
# Contact     : erdogant@gmail.com
# github      : https://github.com/erdogant/pypickle
# Licence     : MIT
# --------------------------------------------------

import pickle
import os


# %% Save pickle file
def save(filepath, var, overwrite=False, fix_imports=True, verbose=3):
    """Save pickle file for input variables.

    Parameters
    ----------
    filepath : str
        Pathname to store pickle files.
    var : {list, object, dataframe, etc}
        Name of list or dict or anything else that needs to be stored.
    fix_imports : bool, (default=True)
        fix_imports are used for compatibility support for pickle stream generated by Python 2. If fix_imports is true, pickle will try to map the old Python 2 names to the new names used in Python 3.
    overwrite : bool, (default=False)
        Overwite file if exists.
    verbose : int, optional
        Show message. A higher number gives more informatie. The default is 3.

    Returns
    -------
    Status of succes : bool [True,False].

    Example
    -------
    >>> import pypickle
    >>> filepath = './temp/tes1t.pkl'
    >>> data = [1,2,3,4,5]
    >>> status = pypickle.save(filepath, data)

    """
    # Make empty pickle file
    if os.path.isdir(filepath):
        if verbose>=3: print('[pypickle] >Warning: filepath should be a file with a path and not directory.' %(filepath))
        return False
    if os.path.isfile(filepath) and not overwrite:
        if verbose>=3: print('[pypickle] File already exists and is not overwritten: [%s]' %(filepath))
        return False

    outfile = open(filepath, 'wb')
    # Write and close
    pickle.dump(var, outfile)
    outfile.close()

    if os.path.isfile(filepath):
        if verbose>=3: print('[pypickle] Pickle file saved: [%s]' %filepath)
        out=True
    else:
        if verbose>=3: print('[pypickle] Pickle file could not be saved: [%s]' %filepath)
    return(out)


# %% Load pickle file
def load(filepath, fix_imports=True, encoding="ASCII", errors="strict", verbose=3):
    """Loading pickle files for input variables.

    Parameters
    ----------
    filepath : str
        Pathname to store pickle files.
    fix_imports : bool, (default=True)
        fix_imports are used for compatibility support for pickle stream generated by Python 2. If fix_imports is true, pickle will try to map the old Python 2 names to the new names used in Python 3.
    encoding : str, (default: "ASCII")
        encoding tell pickle how to decode 8-bit string instances pickled by Python 2.
        The encoding can be "bytes" to read these 8-bit string instances as bytes objects. Using encoding="latin1" is required for unpickling NumPy arrays and instances of datetime, date and time pickled by Python 2.
    errors : str, (default: "strict")
        errors tell pickle how to decode 8-bit string instances pickled by Python 2.
    verbose : int, optional
        Show message. A higher number gives more informatie. The default is 3.

    Returns
    -------
    Status of succes : bool [True,False].

    Example
    -------
    >>> import pypickle
    >>> filepath = 'tes1t.pkl'
    >>> data = [1,2,3,4,5]
    >>> status = pypickle.save(filepath, data)
    >>> # Load file
    >>> data = pypickle.load(filepath)


    """
    out=None
    if os.path.isfile(filepath):
        if verbose>=3: print('[pypickle] Pickle file loaded: [%s]' %filepath)
        pickle_off = open(filepath, "rb")
        out = pickle.load(pickle_off, fix_imports=fix_imports, encoding=encoding, errors=errors)
    else:
        if verbose>=3: print('[pypickle] Pickle file does not exists: [%s]' %filepath)
    return(out)
